#include <fstream>
#include <iostream>
#include "../kdm2/header.h"

// Функція для відкриття файлу та витягнення даних з нього
void read_coefficients(float** matrix, float* constants,std::string filename="input",int n=3) {
    std::ifstream file;
    file.open(filename);
    while (!file.is_open()) {
        std::cout << "File " << filename << " didn't open! Please enter the filename again: ";
        std::cin >> filename;
        file.open(filename);
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++)
            file >> matrix[i][j];
        file >> constants[i];
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++)
            file >> matrix[i][j];
        file >> constants[i];
    }
    file.close();
}

void swap_rows(float** matrix, float* b, int row1, int row2, int n) {
        swap(matrix[row1], matrix[row2]);
    swap(b[row1], b[row2]);
}

void gauss_elimination_demo(float** matrixo, float* bo, int n=3) {
    float* b=new float[n];
    float** matrix=new float*[n];
    for (char i=0; i<n; i++){matrix[i]=new float[n];for (char j=0;j<n;j++)matrix[i][j]=matrixo[i][j];b[i]=bo[i];}
    for (int i = 0; i < n; i++) {
        // Пошук головного елемента у поточному стовпці
        int max_row = i;
        for (int j = i + 1; j < n; j++) {
            if (abs(matrix[j][i]) > abs(matrix[max_row][i]))
                max_row = j;
        }
        
        // Перевірка на сингулярність матриці
        if (matrix[max_row][i] == 0.0) {
            cout << "Matrix is singular" << endl;
            return;
        }
        
        // Обмін поточного рядка з рядком, що містить головний елемент
        swap_rows(matrix, b, i, max_row, n);
        
        // Приведення до верхньої трикутної форми
        for (int j = i + 1; j < n; j++) {
            float factor = matrix[j][i] / matrix[i][i];
            for (int k = i; k < n; k++)
                matrix[j][k] -= matrix[i][k] * factor;
            b[j] -= b[i] * factor;
        }
    }
    
    // Обернений хід
    for (int i = n - 1; i >= 0; i--) {
        for (int j = i + 1; j < n; j++)b[i] -= matrix[i][j] * b[j];
        b[i] /= matrix[i][i];
    }
    for (int i = 0; i < n; i++){cout<<'x'<<i+1<<" = "<<b[i]<<std::endl;delete[] matrix[i];}
    delete[] matrix;
    delete[] b;
}

void lu_decomposition(float** matrix, float* b, int n=3) {
    float** L=new float*[n];float** U=new float*[n];
    for (char i=0; i<n; i++){L[i]=new float[n];U[i]=new float[n];}
    
    // Ініціалізація матриць L та U
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            if (i == j)
                L[i][j] = 1; // Головна діагональ матриці L має значення 1
            else
                L[i][j] = 0;
            U[i][j] = 0;
        }
    }
    
    // Обчислення елементів матриць L та U
    for (int i = 0; i < n; i++) {
        // Обчислення верхньо-трикутної матриці U
        for (int k = i; k < n; k++) {
            float sum = 0;
            for (int j = 0; j < i; j++)
                sum += L[i][j] * U[j][k];
            U[i][k] = matrix[i][k] - sum;
        }
        // Обчислення нижньо-трикутної матриці L
        for (int k = i + 1; k < n; k++) {
            float sum = 0;
            for (int j = 0; j < i; j++)
                sum += L[k][j] * U[j][i];
            L[k][i] = (matrix[k][i] - sum) / U[i][i];
        }
    }
    
    display(U,n,2,"U:");
    display(L,n,2,"L:");
    
    // Розв'язання системи LUx = b
    float y[n];
    for (int i = 0; i < n; i++) {
        float sum = 0;
        for (int j = 0; j < i; j++)
            sum += L[i][j] * y[j];
        y[i] = (b[i] - sum) / L[i][i];
    }
    
    for (int i = n - 1; i >= 0; i--) {
        float sum = 0;
        for (int j = i + 1; j < n; j++)
            sum += U[i][j] * b[j];
        b[i] = (y[i] - sum) / U[i][i];
    }
    for (int i = 0; i < n; i++) {
        cout << "x" << i + 1 << " = " << b[i] << endl;
        delete[] L[i];
        delete[] U[i];
    }
    delete[] L;delete[] U;
}


int main() {
#define n 3
    float** matrix=new float*[n];
    for (char i=0; i<n; i++)matrix[i]=new float[n];
    float* constants=new float[n];
    read_coefficients(matrix,constants);
    display(matrix,n,2,"SLE:",constants);
    cout<<"Розв'язок методом Гауса:"<<std::endl;
    gauss_elimination_demo(matrix,constants,n);
    std::cout<<"nРозв'язок методом LU-розкладу:"<<std::endl;
    lu_decomposition(matrix,constants,n);
    return 0;
    for (char i=0;i<n;i++)delete[] matrix[i];
    delete[] matrix;delete[] constants;
}
